;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xa74a53a92dc635157fe1f305dc526af0 . "./code.cg2js/cg-acache.fasl")
(#x105551c9945f7ee6f9692317fd4e17e1 . "./code.cg2js/cg-base.fasl")
(#x498a6b72624b588a74e6d3119ed4d000 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x164d7091dcbc98d8b783f7f26280fb0c . "./code.cg2js/cg-bitmap-pane.fasl")
(#x13f9709c9129e6e69a8ec1be60187714 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x1c025c3a626e71a94329c26d6bae65dd . "./code.cg2js/cg-button.fasl")
(#xc312de16a53d1960b7fa4270c32b3ed1 . "./code.cg2js/cg-calendar.fasl")
(#x21ee38e9d0e28b028ef2d9d978db4b6f . "./code.cg2js/cg-caret.fasl")
(#x456fb86590ca2331696b45a838be9902 . "./code.cg2js/cg-chart-or-plot.fasl")
(#xe526e9c0664cc5440d58c53767e58659 . "./code.cg2js/cg-chart-widget.fasl")
(#x39b2d6e28cb91a54d55aeb4e5f9b893a . "./code.cg2js/cg-check-box.fasl")
(#x1343c079c22ed2e43bd61762f1270af0 . "./code.cg2js/cg-choice-list.fasl")
(#x35280f33befae2ae64d372b87b72c1a1 . "./code.cg2js/cg-choose-printer.fasl")
(#x53aaa7d645cc662f07f386658a2c82bc . "./code.cg2js/cg-class-grid.fasl")
(#xfb78473e8c5977330c9199213985508c . "./code.cg2js/cg-class-slot-grid.fasl")
(#x42b9046f3edd71e7f16d972490ca46fb . "./code.cg2js/cg-class-support.fasl")
(#x6fcb824684e56e98d857868d6e1e980e . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x82a39923549312af5f567420f4f4ce4f . "./code.cg2js/cg-clipboard-stack.fasl")
(#xea9ea5ad449788ca0d52a1ad93d5342c . "./code.cg2js/cg-clipboard.fasl")
(#x7774e4ca4a0e3192f2a08fc874e816a6 . "./code.cg2js/cg-color-dialog.fasl")
(#xcc7db430824868861a75b7fd78b958c6 . "./code.cg2js/cg-combo-box.fasl")
(#x1da25333c66445bfe4788c27c38a2d27 . "./code.cg2js/cg-common-control.fasl")
(#xafa120227a8fc75edf41ae604111c942 . "./code.cg2js/cg-comtab.fasl")
(#x2b7f683c042e837bc88217b3a56b4637 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x2b0fb87b4564acd6dc3d33726de3c49e . "./code.cg2js/cg-curve.fasl")
(#xce295ffa37048aa5e78828a19d64fe02 . "./code.cg2js/cg-dde-utils.fasl")
(#xb6b3e29b946603d995eedb882fb7dfa5 . "./code.cg2js/cg-dev.fasl")
(#xccd16dfea1a76a13da13f396e549766b . "./code.cg2js/cg-dialog-item.fasl")
(#x2ca497f5840b7f9caa4a2bf257581e19 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x987d6732de1e78e1e4194adbbe627980 . "./code.cg2js/cg-directory-dialog.fasl")
(#x95ac40ed098285d409ffd506369ce4d1 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x63c7ae1bbacb1c1cf2b1ba2c36d20c82 . "./code.cg2js/cg-drag-and-drop.fasl")
(#xb0ebc2c2e4ce201563097249b837a285 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x5368b09ae7eea9c71403926458f92f5b . "./code.cg2js/cg-drawable.fasl")
(#xfeda289b72f74ce09d22060fb005f222 . "./code.cg2js/cg-dropping-outline.fasl")
(#xfb1a5985f9d340ecdc9d3fa2bcf97dd1 . "./code.cg2js/cg-edit-in-place.fasl")
(#x9aeaf506158acd0b1b83354ace268617 . "./code.cg2js/cg-editable-text.fasl")
(#xca402a990c797e583b93320393a50d9d . "./code.cg2js/cg-file-dialog.fasl")
(#x95bbffd628456dfae733434056eb6d66 . "./code.cg2js/cg-fill-texture.fasl")
(#x9df2b788474b74cefa9e950079688c4c . "./code.cg2js/cg-find-string-dialog.fasl")
(#x72c1166c3dfff50c37b95241655e8449 . "./code.cg2js/cg-font-dialog.fasl")
(#x8f6dc1eb46ba5e9b5b8c07c66a60f092 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x7c4d56bc000f1c040df297c18092135e . "./code.cg2js/cg-get-pixmap.fasl")
(#xcdc8156c6d6b7a0dc1fdee2875de6259 . "./code.cg2js/cg-get-position.fasl")
(#xa2db11f20c07dc468292d46e8f8d9d59 . "./code.cg2js/cg-graphics-context.fasl")
(#x64962d171a84697fe37c99f32ac8b11d . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xa47ce5febe9be333eccaa8a7a2085d8b . "./code.cg2js/cg-grid-widget.fasl")
(#xe5cd9175ca2f63abc9bd235f7209bc2a . "./code.cg2js/cg-group-box.fasl")
(#x7776f264d4b83425dadae662f6ad97b7 . "./code.cg2js/cg-header-control.fasl")
(#x9c20496226c57519f1b477352869cfff . "./code.cg2js/cg-hotspot.fasl")
(#xfdea8e643950c12c15e636f303aef281 . "./code.cg2js/cg-html-dialog.fasl")
(#x60d8673c3cbb1097192100e650de7866 . "./code.cg2js/cg-html-widget.fasl")
(#x7da5e1912453819b8dc7627b35b2ee23 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x9ac81743d45fc5d6a444bf61ae9d3fab . "./code.cg2js/cg-icon.fasl")
(#xa51e6d3fed13b6cc22f6df6739dd0470 . "./code.cg2js/cg-ie.fasl")
(#xa4ea8c4b6b182077684a0508525a8001 . "./code.cg2js/cg-intersect-posbox.fasl")
(#xc1bf9495d21a8215923309657bdca8fc . "./code.cg2js/cg-intersect.fasl")
(#xf0b052b181749b48b17d266ffd4b2029 . "./code.cg2js/cg-item-list.fasl")
(#x3b86873c5f004d533cbc7343bd8b3fce . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x2053457889e3b340eeaba94be1998dab . "./code.cg2js/cg-lamp.fasl")
(#x14c5f52e8dab3f9f26afb59107a750b2 . "./code.cg2js/cg-layout.fasl")
(#x0c95de6dcbace4af46f233b986a85d49 . "./code.cg2js/cg-lettered-menu.fasl")
(#x4d0abb267b9f3cd761545ade01330b30 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x9da9f4ba0c3a789c95ad0021a79fbfeb . "./code.cg2js/cg-lisp-text.fasl")
(#xb5a69d56f4b2cdebd134c25efd624c65 . "./code.cg2js/cg-lisp-widget.fasl")
(#xc662266d9204653224bd7ca88eb60b65 . "./code.cg2js/cg-list-view.fasl")
(#x0ce23d3c6d97872780b65ba40915f090 . "./code.cg2js/cg-mci.fasl")
(#x3eb9c40246753a70b02945cabcc9c4a9 . "./code.cg2js/cg-menu-tooltip.fasl")
(#xaa875bb0ade15ca41a5c5b49f9797730 . "./code.cg2js/cg-menu.fasl")
(#x1903fe85a18c048848d607379b8ffe86 . "./code.cg2js/cg-message-dialog.fasl")
(#xe9ccf9f19bdc82bf81b82e39c1365826 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xcf3ddba54f2c76348f5c61c1fc218e62 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xb7b5ddcc58a82896f8e9ffd2d4a3b461 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x75cbbe9a99fad5c297fd54752b33aabb . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x4fe210809fbc277ae88510d8bab8c3b8 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x27def10b7c213cc9f0053fda3701b2e6 . "./code.cg2js/cg-nodes.fasl")
(#xb96b3dc803dbaf67f241ad5dd893173e . "./code.cg2js/cg-object-editor-layout.fasl")
(#x23821aad99ce6f688be54e839eed460f . "./code.cg2js/cg-object-editor.fasl")
(#x737b0d31ba1a2783c1331a1fe7531b16 . "./code.cg2js/cg-ocx.fasl")
(#x2460efb903f132166fcc2e765564def6 . "./code.cg2js/cg-os-widget.fasl")
(#x4b534da7a5c7719201ba5029878363ce . "./code.cg2js/cg-os-window.fasl")
(#xf309515bb4c155d7d30b66a9eb7e34ea . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xaae61fb1d8e19f9924803bf141805631 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xddbc7adde9fbd8513a79e0734cceab4c . "./code.cg2js/cg-outline.fasl")
(#x0fadb5e70f1efb6e0e849158011a7727 . "./code.cg2js/cg-palette.fasl")
(#xa1aaa4f6020be32b8c156ee25c19a4bf . "./code.cg2js/cg-paren-matching.fasl")
(#xd1ea435596ffe9e44be5d61a86ebd848 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x5fe7a91cac2a42d7e0186a1d37fdfc48 . "./code.cg2js/cg-picture-widget.fasl")
(#x721e67990cf0fc4f4b4948055642b00b . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x7b6a422b0e33fc52b326a1f2a73f6220 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x82b61ff19d8e13e1ec0f2a9cb3159d45 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x9285428b639b4cdfedfbd5d7181f04fc . "./code.cg2js/cg-pixmap-widget.fasl")
(#xf937a9fc243c1deb95cec5ff54e28945 . "./code.cg2js/cg-pixmap.fasl")
(#x0f43a409a4173984fb12dd17c29fda26 . "./code.cg2js/cg-plot-widget.fasl")
(#xe230bdd8fc8cb701f9a471e764366f7a . "./code.cg2js/cg-printing.fasl")
(#x1d4bf3788a893a72309d8fcbebe8d814 . "./code.cg2js/cg-progress-indicator.fasl")
(#x6d49ad872a06805bd52605b85d008d5f . "./code.cg2js/cg-project-window.fasl")
(#xf2d2e4e67574a38cf896a42c2d3ea076 . "./code.cg2js/cg-property.fasl")
(#x91f8a9901546896961194f216d53a1a9 . "./code.cg2js/cg-radio-button.fasl")
(#x06fdf9f0816373166eaf0eff81fc2c98 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x5797e17f155f15f7e92cb01d905e1bbc . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x8a2e90fb6bf9e3fe8d838c62e5766ffc . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x93188badac3523dbebbba5d1c10f218a . "./code.cg2js/cg-rich-edit.fasl")
(#x43cc45dbc23add7083fe7d41a3ec6aab . "./code.cg2js/cg-sample-file-menu.fasl")
(#x1725808f9ca5d857f673d906fef60532 . "./code.cg2js/cg-scaling-stream.fasl")
(#x7dda7b33f2948511376d235cd55eb850 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xfa55fc95a42685bfc25260d71ad2bcb3 . "./code.cg2js/cg-scroll-bar.fasl")
(#x53e1d3ee18afa207db09f87ca77d02c6 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xc11adfb21814c8727873d5f4b65b0b73 . "./code.cg2js/cg-selected-object.fasl")
(#xc200ec708640834b9cee91c60deca074 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x5b110c703b26e81759ef9575d4d5e06b . "./code.cg2js/cg-split-bar.fasl")
(#xd00dca47c774623fd6bc933972ebc0b9 . "./code.cg2js/cg-static-text.fasl")
(#x136815b67119cbe5adeac968cc9d7073 . "./code.cg2js/cg-status-bar.fasl")
(#x846ea780f0d931d2478c6c98de31c9c1 . "./code.cg2js/cg-string-dialog.fasl")
(#x049312d9f4d6b149965910f20117600b . "./code.cg2js/cg-tab-control.fasl")
(#xc2b627f1ca3aae6148de99d50834bbb8 . "./code.cg2js/cg-template-string.fasl")
(#x5d8a2eb2ed8c86f817384282a4059f9a . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x4afcdf7bb01c3abf63efc35154b7a460 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x26c492a39868fbbe42d0d41a27ab3535 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x433eeaad6133cb8566c1b58dab98d4a9 . "./code.cg2js/cg-text-or-combo.fasl")
(#xc188b362090168883cff719982aae3b2 . "./code.cg2js/cg-text-widget.fasl")
(#x7dc6ce86bfb6245ce7e1a615e9039e5a . "./code.cg2js/cg-timer.fasl")
(#x8a1a6be0de7c721b38e0fc2c7e9de07a . "./code.cg2js/cg-toggling-widget.fasl")
(#xf0b145a1854419e4ac439bed97289da9 . "./code.cg2js/cg-toolbar.fasl")
(#x595df143c702ed90bf30177dc81485f2 . "./code.cg2js/cg-tooltip.fasl")
(#xea440f3b1c0d819c011ba261e3b7c273 . "./code.cg2js/cg-touch.fasl")
(#x071299fb2c8f4bab7c45b1c688d3266a . "./code.cg2js/cg-trackbar.fasl")
(#x7eff2e42106588e67d918040d0d5d0b1 . "./code.cg2js/cg-tray.fasl")
(#x173f1f8744da9360bbb6420fc17d57cc . "./code.cg2js/cg-up-down-control.fasl")
(#x83c404b69405ee60dfcca51a51db84da . "./code.cg2js/cg-utility-dialog.fasl")
(#x4c0a6f878ae73ace4dea512565f4fee3 . "./code.cg2js/cg-web-browser-dde.fasl")
(#xbf76ab03e2ff4c49c834b8ca2360e133 . "./code.cg2js/cg-web-browser.fasl")
(#x9c2e272ab4da56e91acc50fa48b701d6 . "./code.cg2js/cg-wrap-string.fasl")
(#x6cc23239d2d7e312fe88975130677c04 . "./code.cg2js/cg-yes-no-list.fasl")
(#x4fa5a64ebdfe167f363ab54e5fd69d01 . "./code.cg2js/cg-yes-no-string.fasl")
(#x82e4a8a896eca43819a51658db42d313 . "./code.cg2js/cg.fasl")
(#x65a45c6525c8553a87315e7727296de5 . "./code.idejs/ide-acache.fasl")
(#x8b971f4798762e07c1ce8273360c394b . "./code.idejs/ide-apropos.fasl")
(#xefc29c8f29f181910cecc4fb849133b5 . "./code.idejs/ide-base.fasl")
(#xdb720dd97ef03c0ec31da0f7ca29bdee . "./code.idejs/ide-class-browser.fasl")
(#x6889926814bd4a72aecc23b86e6c09df . "./code.idejs/ide-class-grapher.fasl")
(#x0c544b4749d01b27a62e3f5f08075fb4 . "./code.idejs/ide-clipboard.fasl")
(#x04d28b4f4b0f7ff3eb7ec4643e56fac9 . "./code.idejs/ide-dev.fasl")
(#xfc826c44c2de7334b263d011015099a7 . "./code.idejs/ide-editor.fasl")
(#xdad0f22c1b81f8ce377c4a166f70cb4c . "./code.idejs/ide-find-definitions.fasl")
(#x66f0ffeac7cc53964322f9b1a76786d0 . "./code.idejs/ide-find-in-files.fasl")
(#x7e43d8ebedd18ea058d2603bd7f733c7 . "./code.idejs/ide-form.fasl")
(#x6a64b6c22b4f44a85acea41014208f29 . "./code.idejs/ide-grapher.fasl")
(#xdb773e426ef0a645faa1e565dcc80a97 . "./code.idejs/ide-inspector.fasl")
(#x680e4b6e45a57924a210df58ebeb755d . "./code.idejs/ide-intro.fasl")
(#x52ea07175373486cdc2a4d0a0ef85ef7 . "./code.idejs/ide-keyshort.fasl")
(#xd1c58e57fada8c0ba514e08f683e5e47 . "./code.idejs/ide-navigator.fasl")
(#xaa823f115999e3b6f996ede8e543062c . "./code.idejs/ide-package-list.fasl")
(#x3eec2d73878d7806d8c7ca2258ec9356 . "./code.idejs/ide-patch-dialog.fasl")
(#xdee02de948998a5d0e167b05eaed9399 . "./code.idejs/ide-process-browser.fasl")
(#xb87b5a71cab4baa1b7c4a834db4f1457 . "./code.idejs/ide-project.fasl")
(#xf3cc70f5972c6d0ae91c1ad7c9c72743 . "./code.idejs/ide-trace.fasl")
(#x4d709f9d8fc84ee734db4380f82df521 . "./code.idejs/ide-window-list.fasl")
(#xceb794d2a93c555b945d7aeebce8e467 . "./code.idejs/ide.fasl")
(#xdf73f3983b983ba8030e7ee4702b498a . "./cgjs.html")
(#xa0fd27890288fb85b9ee621ed47b2251 . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
